@@ -1,9 +1,10 @@
 import collections.abc
+import contextlib
 import queue
 import time
 import warnings
 from typing import (
-    Any, Dict, Iterable, List, Mapping, Optional, Tuple, Union, TYPE_CHECKING
+    Any, Dict, Generator, Iterable, List, Mapping, Optional, Tuple, Union, TYPE_CHECKING
 )
 
 import torch
@@ -23,6 +24,12 @@ if TYPE_CHECKING:
     from pytorch_pfn_extras.profiler._time_summary import _ReportNotification
 
 
+@contextlib.contextmanager
+def _nullcontext() -> Generator[None, None, None]:
+    # contextlib.nullcontext equivalent, needed for Python 3.6 support.
+    yield
+
+
 class Trainer:
     def __init__(
             self,
@@ -32,6 +39,7 @@ class Trainer:
                 'Evaluator', Tuple['Evaluator', TriggerLike],
                 Mapping[str, Union['Evaluator', Tuple['Evaluator', TriggerLike]]]]],
             models: Union[torch.nn.Module, Mapping[str, torch.nn.Module]],
+            profile: Optional[torch.profiler.profile] = None,  # type: ignore[name-defined]
             **kwargs: Any,
     ):
         self.handler = handler
@@ -46,7 +54,8 @@ class Trainer:
         else:
             self._models = models
         self._kwargs = kwargs
-        self._enable_profile = kwargs.get('enable_profile', False)
+        self._profile = profile
+        self._enable_profile = kwargs.get('enable_profile', profile is not None)
         self._extensions: List[  # list of (args, kwargs)
             Tuple[Tuple[
                 Union['extension.ExtensionLike', extension.ExtensionEntry],
@@ -159,10 +168,7 @@ class Trainer:
             self,
             idx: int,
             outs: Any,
-            *,
-            is_deferred: bool = False,
     ) -> None:
-        self._deferred = False  # notify that the function was called
         c_idx = self._idxs.get()
         # Asure that iterations complete in order
         if c_idx != idx:
@@ -173,32 +179,13 @@ class Trainer:
             )
         x = self._inputs.get()
         begin = self._times.get()
-        observed = self._observed.get()
         (
             record_iteration,
             record_run_iteration,
             record_train_step,
         ) = self._profile_records.get()
-        # If the iteration was not deferred this is still under the
-        # `manager.run_iteration` scope
-        # Change the current reporter observation
-        # To be the one to be completed
-        if is_deferred:
-            # Complete profiler record of `train_step`
-            record_train_step.complete()
-            # We want to report the previously obtained values in `train_step`
-            cm_iter = self.manager.complete_iteration(observation=observed)
-            cm_iter.__enter__()
-        else:
-            reporting.get_current_reporter().observation = observed
-            self.manager.observation = observed
         self.handler.train_post_step(self, idx, x, outs)
         reporting.report({"elapsed_time": time.time() - begin})
-        if is_deferred:
-            cm_iter.__exit__(None, None, None)
-            # Complete profiler record of `run_iteration` and iteration
-            record_run_iteration.complete()
-            record_iteration.complete()
 
     def run(self,
             train_loader: Iterable[Any],
@@ -276,79 +263,83 @@ class Trainer:
                 for _, (evaluator, _) in self._evaluators.items():
                     evaluator.handler.eval_setup(evaluator, val_loader)
 
-        while not self.manager.stop_trigger:
-            self.handler.train_epoch_begin(self, train_loader)
+        with self._profile or _nullcontext() as prof:
+            while not self.manager.stop_trigger:
+                self.handler.train_epoch_begin(self, train_loader)
 
-            # When iterations are completed in the callback
-            # This is needed to avoid being constantly passing parameters
-            self._idxs: 'queue.Queue[int]' = queue.Queue()
-            self._inputs: 'queue.Queue[Any]' = queue.Queue()
-            self._times: 'queue.Queue[float]' = queue.Queue()
-            self._observed: 'queue.Queue[reporting.Observation]' = queue.Queue()
-            # Iterator must be created after `train_epoch_begin` as it may be
-            #  using a DistributedSampler.
-            loader_iter = iter(train_loader)
-            self._profile_records: 'queue.Queue[List[_ReportNotification]]' \
-                = queue.Queue()
-            for idx in range(train_len):
-                with record(
-                    "pytorch_pfn_extras.training.Trainer:iteration",
-                    use_cuda=torch.cuda.is_available(),
-                    enable=self._enable_profile
-                ) as ntf0:
-                    try:
-                        with record(
-                            "pytorch_pfn_extras.training.Trainer:get_data",
-                            enable=self._enable_profile
-                        ):
-                            x = next(loader_iter)
-                    except StopIteration:
-                        loader_iter = iter(train_loader)
-                        with record(
-                            "pytorch_pfn_extras.training.Trainer:get_data",
-                            enable=self._enable_profile
-                        ):
-                            x = next(loader_iter)
-                    begin = time.time()
-                    self._idxs.put(idx)
-                    self._inputs.put(x)
-                    self._times.put(begin)
-                    self._deferred = True
+                # When iterations are completed in the callback
+                # This is needed to avoid being constantly passing parameters
+                self._idxs: 'queue.Queue[int]' = queue.Queue()
+                self._inputs: 'queue.Queue[Any]' = queue.Queue()
+                self._times: 'queue.Queue[float]' = queue.Queue()
+                self._observed: 'queue.Queue[reporting.Observation]' = queue.Queue()
+                # Iterator must be created after `train_epoch_begin` as it may be
+                #  using a DistributedSampler.
+                loader_iter = iter(train_loader)
+                self._profile_records: 'queue.Queue[List[_ReportNotification]]' \
+                    = queue.Queue()
+                for idx in range(train_len):
                     with record(
-                        "pytorch_pfn_extras.training.Trainer:run_iteration",
+                        "pytorch_pfn_extras.training.Trainer:iteration",
                         use_cuda=torch.cuda.is_available(),
                         enable=self._enable_profile
-                    ) as ntf1, \
-                            self.manager.run_iteration() as iter_notifier:
-                        self._observed.put(self.manager.observation)
-                        with record(
-                            "pytorch_pfn_extras.training.Trainer:train_step",
-                            use_cuda=torch.cuda.is_available(),
-                            enable=self._enable_profile
-                        ) as ntf2:
-                            self._profile_records.put([ntf0, ntf1, ntf2])
-                            self.handler.train_step(
-                                self, idx, x, complete_fn=self._complete_step)
-                            # Check if the callback was called
-                            if self._deferred:
-                                # The iteration will be completed later
-                                ntf0.defer()
-                                ntf1.defer()
-                                ntf2.defer()
-                                iter_notifier.defer()
+                    ) as ntf0:
+                        try:
+                            with record(
+                                "pytorch_pfn_extras.training.Trainer:get_data",
+                                enable=self._enable_profile
+                            ):
+                                x = next(loader_iter)
+                        except StopIteration:
+                            loader_iter = iter(train_loader)
+                            with record(
+                                "pytorch_pfn_extras.training.Trainer:get_data",
+                                enable=self._enable_profile
+                            ):
+                                x = next(loader_iter)
+                        begin = time.time()
+                        self._idxs.put(idx)
+                        self._inputs.put(x)
+                        self._times.put(begin)
+                        try:
+                            with record(
+                                "pytorch_pfn_extras.training.Trainer:run_iteration",
+                                use_cuda=torch.cuda.is_available(),
+                                enable=self._enable_profile
+                            ) as ntf1, \
+                                    self.manager.run_iteration():
+                                self._observed.put(self.manager.observation)
+                                with record(
+                                    "pytorch_pfn_extras.training.Trainer:train_step",
+                                    use_cuda=torch.cuda.is_available(),
+                                    enable=self._enable_profile
+                                ) as ntf2:
+                                    self._profile_records.put([ntf0, ntf1, ntf2])
+                                    self.handler.train_step(
+                                        self, idx, x, complete_fn=self._complete_step)
+                                    # Check if the callback was called
+                        except Exception:
+                            # The manager has errored and called the extensions
+                            # on_error. However the manager is reusable
+                            # so training can continue and extensions state is not
+                            # finalized. On the other hand, the trainer is not
+                            # reusable, so we finalize the extensions here.
+                            self.manager.finalize()
+                            raise
+
+                    if prof is not None:
+                        prof.step()  # type: ignore[no-untyped-call]
                     # In some cases, DataLoaders are continuos
                     # And will keep yielding results even if the epoch
                     # is completed. We forcefully exit at the end of
                     # every epoch
-                    if (
-                        self.is_epoch_last_iter(idx)
-                        or self.manager.stop_trigger
-                    ):
+                    if self.is_epoch_last_iter(idx) or self.manager.stop_trigger:
                         break
-            # In handlers that support a completely Async model train_epoch_end
-            # Will take care of completing pending work
-            self.handler.train_epoch_end(self)
-
+                # In handlers that support a completely Async model train_epoch_end
+                # Will take care of completing pending work
+                self.handler.train_epoch_end(self)
+            if prof is not None:
+                prof.on_trace_ready = None
         self.handler.train_cleanup(self)
 
 
